@@ -1,4 +1,4 @@
-/* $OpenBSD: x509_set.c,v 1.9 2014/07/11 08:44:49 jsing Exp $ */
+/* $OpenBSD: x509_set.c,v 1.12 2015/09/30 17:49:59 jsing Exp $ */
 /* Copyright (C) 1995-1998 Eric Young (eay@cryptsoft.com)
  * All rights reserved.
  *
@@ -69,7 +69,7 @@ X509_set_version(X509 *x, long version)
 	if (x == NULL)
 		return (0);
 	if (x->cert_info->version == NULL) {
-		if ((x->cert_info->version = M_ASN1_INTEGER_new()) == NULL)
+		if ((x->cert_info->version = ASN1_INTEGER_new()) == NULL)
 			return (0);
 	}
 	return (ASN1_INTEGER_set(x->cert_info->version, version));
@@ -84,9 +84,9 @@ X509_set_serialNumber(X509 *x, ASN1_INTEGER *serial)
 		return (0);
 	in = x->cert_info->serialNumber;
 	if (in != serial) {
-		in = ASN1_STRING_dup(serial);
+		in = ASN1_INTEGER_dup(serial);
 		if (in != NULL) {
-			M_ASN1_INTEGER_free(x->cert_info->serialNumber);
+			ASN1_INTEGER_free(x->cert_info->serialNumber);
 			x->cert_info->serialNumber = in;
 		}
 	}
@@ -120,7 +120,7 @@ X509_set_notBefore(X509 *x, const ASN1_TIME *tm)
 	if (in != tm) {
 		in = ASN1_STRING_dup(tm);
 		if (in != NULL) {
-			M_ASN1_TIME_free(x->cert_info->validity->notBefore);
+			ASN1_TIME_free(x->cert_info->validity->notBefore);
 			x->cert_info->validity->notBefore = in;
 		}
 	}
@@ -138,7 +138,7 @@ X509_set_notAfter(X509 *x, const ASN1_TIME *tm)
 	if (in != tm) {
 		in = ASN1_STRING_dup(tm);
 		if (in != NULL) {
-			M_ASN1_TIME_free(x->cert_info->validity->notAfter);
+			ASN1_TIME_free(x->cert_info->validity->notAfter);
 			x->cert_info->validity->notAfter = in;
 		}
 	}
