@@ -75,6 +75,12 @@
 
 #define BUF_MAX 1024
 
+#ifdef _MSC_VER
+#ifdef IN
+#undef IN
+#endif
+#endif
+
 /* These are the different types of line that are found in the input file. */
 enum {
 	AEAD = 0,	/* name of the AEAD algorithm. */
@@ -134,11 +140,11 @@ aead_from_name(const EVP_AEAD **aead, const char *name)
 #else
 		fprintf(stderr, "No chacha20-poly1305 support.\n");
 #endif
-	} else if (strcmp(name, "chacha20-poly1305-ietf") == 0) {
+	} else if (strcmp(name, "chacha20-poly1305-old") == 0) {
 #if !defined(OPENSSL_NO_CHACHA) && !defined(OPENSSL_NO_POLY1305)
-		*aead = EVP_aead_chacha20_poly1305_ietf();
+		*aead = EVP_aead_chacha20_poly1305_old();
 #else
-		fprintf(stderr, "No chacha20-poly1305-ietf support.\n");
+		fprintf(stderr, "No chacha20-poly1305-old support.\n");
 #endif
 	} else {
 		fprintf(stderr, "Unknown AEAD: %s\n", name);
@@ -199,6 +205,11 @@ run_test_case(const EVP_AEAD* aead, unsigned char bufs[NUM_TYPES][BUF_MAX],
 		return 0;
 	}
 
+	if (memcmp(out2, bufs[IN], out_len2) != 0) {
+		fprintf(stderr, "Plaintext mismatch on line %u\n", line_no);
+		return 0;
+	}
+
 	out[0] ^= 0x80;
 	if (EVP_AEAD_CTX_open(&ctx, out2, &out_len2, lengths[IN], bufs[NONCE],
 	    lengths[NONCE], out, out_len, bufs[AD], lengths[AD])) {
